# --------------------------------------------------------------
# TASKMAP UNIT TESTS
# To be used in the Jenkins build pipeline for VITO-PyREST
# --------------------------------------------------------------
# IMPORTS AND LIBRARIES
# --------------------------------------------------------------
#from __future__ import absolute_import
import unittest
from db.wigldap import WigLDAP
from db.database import WigFields
from ..utils.taskmap import Taskmap
from os.path import join
import requests
import os
import binascii
import json
import sys

import logging

# --------------------------------------------------------------
# VARIABLES
# --------------------------------------------------------------
log = logging.getLogger(__name__)
log.setLevel(logging.DEBUG)
ldap = WigLDAP()

# Change these according to the map type. For shadowmaps, these have no effect.
stepvalue = 0.1
maxvalue = 1.15
minvalue = 0.85
avg_greenness = 0
product_id = "S2_FAPAR"
start_date = '23-6-2019'
end_date = '23-6-2019'

# --------------------------------------------------------------
# MAIN TEST FUNCTIONS
# --------------------------------------------------------------
def main_test(login, workdir, pyrestserver):
    wdir = join(workdir, str(binascii.hexlify(os.urandom(16))).replace('\'', ''))  # Generates a random foldername
    result_list = []
    src_fields = fetch_fields(login)


    for field in src_fields:
        log.debug("Processing field %s " % field["metadata"]["fieldName"])
        geometry = field["metadata"]["geometry"]

        jsondata = {}
        jsondata['timestamp'] = ''
        jsondata['geometry'] = geometry
        jsondata['stepvalue'] = stepvalue
        jsondata['maxvalue'] = maxvalue
        jsondata['minvalue'] = minvalue
        jsondata['avg_greenness'] = avg_greenness

        jsonstr = json.dumps(jsondata)

        try:
            log.debug("Fetching baseline data from pyrest-dev...")
            headers = {'Content-type': 'application/json'}
            api_request = requests.post('http://{0}:8080/taskmap/generate/shadow'.format(pyrestserver), data=jsonstr, headers=headers, timeout=5)
            if api_request.status_code == 400:
                log.debug("No data found for this field. Data returned: {0}".format(api_request.text))
                old_json = "Error400"
                new_json = "Error400"
                result_list.append(True)
            else:
                log.debug("Field data found. Proceeding...")
                old_json = json.loads(api_request.content)
                new_json = json.dumps(Taskmap.render_shadowmap_from_geojson(jsondata, wdir))
                if old_json == new_json:
                    result_list.append(True)

        except Exception as e:
            log.error("Test failed: {0} - Line: {1}".format(e, sys.exc_info()[-1].tb_lineno))
            return False

        except ValueError as e:
            log.error("JSON is invalid. Exception: {0} - Line: {1} - Comparing strings instead...".format(e, sys.exc_info()[-1].tb_lineno))
            if old_json == new_json:
                result_list.append(True)
            else:
                result_list.append(False)
        except Exception as e:
            log.error("Test failed on line {0}: {1}".format(sys.exc_info()[-1].tb_lineno, e))


    if len(result_list) > 0:
        return all(result_list)
    else:
        return True


def fetch_fields(useremail):
    # Retrieve user ID via LDAP
    uid = ldap.getInfoByMail(useremail)['uid'][0].decode('utf-8')
    # Retrieve the fields of the user
    src_fields = list(map(lambda x: x["_source"], WigFields.getParcelsByUser(uid)))
    logging.debug("Found %s parcels for user %s" % (len(src_fields), uid))

    return src_fields

if __name__ == '__main__':
    unittest.main()